import datetime
from typing import NamedTuple, List, Optional

from pscr_common.api.models import (
    with_swagger_definition,
    with_from_dict,
    ServerProvided,
)

from .helpers import ValidationError

# noinspection PyUnresolvedReferences
from .feed_models import (
    Report,
    IOCs,
    IOC_V2,
    QueryIOC,
)  # import these so they show up in the swagger docs


@with_swagger_definition
@with_from_dict
class ReportSeverity(NamedTuple):
    report_id: str
    severity: int


@with_swagger_definition
@with_from_dict
class ClassifierWatchlist(NamedTuple):
    name: str
    classifier_key: str
    classifier_value: str
    description: Optional[str] = None
    watchlist_id: ServerProvided[str] = None
    tags_enabled: Optional[bool] = None
    alerts_enabled: Optional[bool] = None
    alert_classification_enabled: Optional[bool] = None
    create_timestamp: ServerProvided[datetime.datetime] = None
    last_update_timestamp: ServerProvided[datetime.datetime] = None


@with_swagger_definition
@with_from_dict
class ReportWatchlist(NamedTuple):
    name: str
    report_ids: List[str]
    description: Optional[str] = None
    watchlist_id: ServerProvided[str] = None
    tags_enabled: Optional[bool] = None
    alerts_enabled: Optional[bool] = None
    alert_classification_enabled: Optional[bool] = None
    create_timestamp: ServerProvided[datetime.datetime] = None
    last_update_timestamp: ServerProvided[datetime.datetime] = None


@with_swagger_definition
@with_from_dict
class Watchlist(NamedTuple):
    classifier: Optional[ClassifierWatchlist] = None
    report: Optional[ReportWatchlist] = None

    @staticmethod
    def _validate(watchlist):
        if not watchlist.classifier and not watchlist.report:
            raise ValidationError("Must provide 'classifier' or 'report' watchlist")
        if watchlist.classifier and watchlist.report:
            raise ValidationError("Cannot provide both 'classifier' and 'report'")
        if watchlist.classifier and not watchlist.classifier.name:
            raise ValidationError(
                "Must provide non-empty 'name' for classifier watchlist"
            )
        if watchlist.report and not watchlist.report.name:
            raise ValidationError("Must provide non-empty 'name' for report watchlist")

    def to_v2(self):
        if self.classifier:
            return WatchlistV2(
                id=self.classifier.watchlist_id,
                name=self.classifier.name,
                description=self.classifier.description,
                tags_enabled=self.classifier.tags_enabled,
                alerts_enabled=self.classifier.alerts_enabled,
                create_timestamp=self.classifier.create_timestamp,
                last_update_timestamp=self.classifier.last_update_timestamp,
                classifier=ClassifierKeyValue(
                    self.classifier.classifier_key, self.classifier.classifier_value
                ),
                alert_classification_enabled=self.classifier.alert_classification_enabled,
            )
        else:
            return WatchlistV2(
                id=self.report.watchlist_id,
                name=self.report.name,
                description=self.report.description,
                tags_enabled=self.report.tags_enabled,
                alerts_enabled=self.report.alerts_enabled,
                create_timestamp=self.report.create_timestamp,
                last_update_timestamp=self.report.last_update_timestamp,
                report_ids=self.report.report_ids,
                alert_classification_enabled=self.report.alert_classification_enabled,
            )


@with_swagger_definition
@with_from_dict
class ClassifierKeyValue(NamedTuple):
    key: str
    value: str


@with_swagger_definition
@with_from_dict
class WatchlistV2(NamedTuple):
    name: str
    description: Optional[str] = None
    id: ServerProvided[str] = None
    tags_enabled: Optional[bool] = None
    alerts_enabled: Optional[bool] = None
    alert_classification_enabled: Optional[bool] = None
    create_timestamp: ServerProvided[datetime.datetime] = None
    last_update_timestamp: ServerProvided[datetime.datetime] = None
    report_ids: Optional[List[str]] = None
    classifier: Optional[ClassifierKeyValue] = None

    @staticmethod
    def _validate(watchlist):
        if not watchlist.name:
            raise ValidationError("Must provide non-empty 'name' for watchlist")

    def to_v1(self):
        if self.classifier:
            return Watchlist(
                classifier=ClassifierWatchlist(
                    watchlist_id=self.id,
                    name=self.name,
                    description=self.description,
                    tags_enabled=self.tags_enabled,
                    alerts_enabled=self.alerts_enabled,
                    alert_classification_enabled=self.alert_classification_enabled,
                    create_timestamp=self.create_timestamp,
                    last_update_timestamp=self.last_update_timestamp,
                    classifier_key=self.classifier.key,
                    classifier_value=self.classifier.value,
                )
            )
        else:
            return Watchlist(
                report=ReportWatchlist(
                    watchlist_id=self.id,
                    name=self.name,
                    description=self.description,
                    tags_enabled=self.tags_enabled,
                    alerts_enabled=self.alerts_enabled,
                    alert_classification_enabled=self.alert_classification_enabled,
                    create_timestamp=self.create_timestamp,
                    last_update_timestamp=self.last_update_timestamp,
                    report_ids=self.report_ids,
                )
            )


@with_swagger_definition
@with_from_dict
class ReportIOCIgnore(NamedTuple):
    ignore: bool
    report_id: str
    ioc_id: Optional[str] = None


@with_swagger_definition
@with_from_dict
class ReportIOCIgnoreList(NamedTuple):
    ignores: List[ReportIOCIgnore]


@with_swagger_definition
@with_from_dict
class WatchlistTelemetry(NamedTuple):
    watchlist_id: str
    interval: int
    hits: int
    executions: int


@with_swagger_definition
@with_from_dict
class WatchlistTelemetryList(NamedTuple):
    telemetry: List[WatchlistTelemetry]
