import re

PREFIX = "##"
QUOTED_PATTERN = re.compile('(\\w+:)("(?:\\\\"|.)*?")')
RANGE_PATTERN = re.compile("(\\w+:)(\\[.+?\\])")
TERM_PATTERN = re.compile("(\\w+:)((?:\\\\ |\\\\\\)|[^\\s)])*)")
PRE_ESCAPED_IN_TERMS = frozenset([" ", '"', ")", "(", ":"])
PRE_ESCAPED_IN_QUOTED_TERMS = frozenset([" ", '"'])
PRE_ESCAPED_IN_CMDLINE = frozenset(["\\", "*", "?"])
FIELD_MAP = dict(
    [
        ("digsig_result", "process_publisher_state"),
        ("digsig_result_parent", "parent_publisher_state"),
        ("digsig_result_child", "childproc_publisher_state"),
        ("digsig_result_modload", "modload_publisher_state"),
        ("digsig_result_filemod", "filemod_publisher_state"),
        ("digsig_publisher", "process_publisher"),
        ("unique_id", "document_guid"),
        ("id", "process_guid"),
        ("parent_id", "parent_guid"),
        ("sensor_id", "device_id"),
        ("ipaddr", "netconn_ipv4"),
        ("ipv6addr", "netconn_ipv6"),
        ("process_name", "process_name"),
        ("parent_name", "parent_name"),
        ("childproc_name", "childproc_name"),
        ("crossproc_name", "crossproc_name"),
        ("hostname", "device_name"),
        ("os_type", "device_os"),
        ("group", "device_group"),
        ("process_md5", "process_hash"),
        ("parent_md5", "parent_hash"),
        ("filewrite_md5", "filemod_hash"),
        ("childproc_md5", "childproc_hash"),
        ("crossproc_md5", "crossproc_hash"),
        ("blocked_md5", "blocked_hash"),
        ("md5", "hash"),
        ("crossproc_type", "crossproc_action"),
        ("block_status", "sensor_action"),
        ("path", "process_name"),  # Map path to process name
        ("modload", "modload_name"),
        ("filemod", "filemod_name"),
        ("regmod", "regmod_name"),
        ("modload_count", "modload_count"),
        ("filemod_count", "filemod_count"),
        ("regmod_count", "regmod_count"),
        ("netconn_count", "netconn_count"),
        ("childproc_count", "childproc_count"),
        ("crossproc_count", "crossproc_count"),
        # ("processblock_count",      ""), # TODO: Maybe replace with sensor_action:BLOCK
        ("process_pid", "process_pid"),
        ("parent_pid", "parent_pid"),
        ("ipport", "netconn_port"),
        ("server_added_timestamp", "backend_timestamp"),
        ("last_update", "device_timestamp"),
        ("last_server_update", "backend_timestamp"),
        ("terminated", "process_terminated"),
        ("domain", "netconn_domain"),
        ("cmdline", "process_cmdline"),
        ("username", "process_username"),
        ("product_version", "process_product_version"),
        ("original_filename", "process_original_filename"),
        ("file_desc", "process_file_description"),
        ("internal_name", "process_internal_name"),
        ("company_name", "process_company_name"),
        ("product_name", "process_product_name"),
        ("fileless_scriptload_cmdline", "fileless_scriptload_cmdline"),
        ("fileless_scriptload_cmdline_length", "fileless_scriptload_cmdline_length"),
        ("fileless_scriptload_hash", "fileless_scriptload_hash"),
    ]
)

FIELD_VALUE_MAP = dict(
    [
        (
            "digsig_result",
            {
                "signed": "FILE_SIGNATURE_STATE_TRUSTED",
                "unsigned": "FILE_SIGNATURE_STATE_NOT_SIGNED",
                "bad signature": "!",
                "invalid signature": "!",
                "expired": "!",
                "invalid chain": "!",
                "untrusted root": "!",
                "explicit distrust": "!",
            },
        ),
        (
            "digsig_result_parent",
            {
                "signed": "FILE_SIGNATURE_STATE_TRUSTED",
                "unsigned": "FILE_SIGNATURE_STATE_NOT_SIGNED",
                "bad signature": "!",
                "invalid signature": "!",
                "expired": "!",
                "invalid chain": "!",
                "untrusted root": "!",
                "explicit distrust": "!",
            },
        ),
        (
            "digsig_result_child",
            {
                "signed": "FILE_SIGNATURE_STATE_TRUSTED",
                "unsigned": "FILE_SIGNATURE_STATE_NOT_SIGNED",
                "bad signature": "!",
                "invalid signature": "!",
                "expired": "!",
                "invalid chain": "!",
                "untrusted root": "!",
                "explicit distrust": "!",
            },
        ),
        (
            "digsig_result_modload",
            {
                "signed": "FILE_SIGNATURE_STATE_TRUSTED",
                "unsigned": "FILE_SIGNATURE_STATE_NOT_SIGNED",
                "bad signature": "!",
                "invalid signature": "!",
                "expired": "!",
                "invalid chain": "!",
                "untrusted root": "!",
                "explicit distrust": "!",
            },
        ),
        (
            "digsig_result_filemod",
            {
                "signed": "FILE_SIGNATURE_STATE_TRUSTED",
                "unsigned": "FILE_SIGNATURE_STATE_NOT_SIGNED",
                "bad signature": "!",
                "invalid signature": "!",
                "expired": "!",
                "invalid chain": "!",
                "untrusted root": "!",
                "explicit distrust": "!",
            },
        ),
        ("os_type", {"osx": "MAC", "windows": "WINDOWS", "linux": "LINUX"}),
        # Can't translate these crossproc_type fields, have to add crossproc_target to query.
        (
            "crossproc_type",
            {
                "processopen": "!",
                "remotethread": "!",
                "processopentarget": "!",
                "remotethreadtarget": "!",
            },
        ),
        # Most of these don't translate 1 to 1
        (
            "block_status",
            {
                "processterminated": "TERMINATE",
                "notterminatedcbprocess": "!",
                "notterminatedsystemprocess": "!",
                "notterminatedcriticalsystemprocess": "!",
                "notterminatedwhitelistedpath": "!",
                "notterminatedopenprocesserror": "!",
                "notterminatedterminateerror": "!",
            },
        ),
    ]
)


def response_field_to_pscr(field):
    fieldNoColon = field[: len(field) - 1]
    retval = FIELD_MAP.get(fieldNoColon)
    if retval is not None:
        return retval + ":"
    raise ValueError(f"Cannot convert query containing field: {fieldNoColon}")


def response_field_value_to_pscr(field, value):
    fieldNoColon = field[: len(field) - 1]
    value_map = FIELD_VALUE_MAP.get(fieldNoColon)
    if value_map is not None:
        was_quoted = False
        if len(value) > 2 and value[0] == '"' and value[len(value) - 1] == '"':
            unquoted_value = value[1 : len(value) - 1]
            was_quoted = True
        else:
            unquoted_value = value

        mapped_value = value_map.get(unquoted_value.lower())
        if mapped_value is not None:
            if mapped_value == "!":
                raise ValueError(
                    f"Cannot convert query containing field: {fieldNoColon} with value {unquoted_value}"
                )
            else:
                if was_quoted:
                    return '"' + mapped_value + '"'
                return mapped_value
        else:
            return value  # Return as is even if invalid, may be query is written to check values for both
    else:
        return value


def extract_matches(query, pattern, matches, counter, escape_function, map_values):
    pos = 0
    sb = ""
    m = pattern.search(query, pos)
    while m:
        key = ""
        while key == "" or key in query:
            key = PREFIX + str(counter) + PREFIX
            counter = counter + 1
        if map_values:
            mapped_value = response_field_value_to_pscr(m.group(1), m.group(2))
        else:
            mapped_value = m.group(2)
        matches[key] = response_field_to_pscr(m.group(1)) + escape_function(
            mapped_value, m.group(1) == "cmdline:"
        )
        sb = sb + query[pos : m.start()] + key
        pos = m.end()
        m = pattern.search(query, pos)
    sb = sb + query[pos:]
    return sb, counter


def escape_term(s, is_cmdline):
    if is_cmdline:
        return escape(s, PRE_ESCAPED_IN_TERMS | PRE_ESCAPED_IN_CMDLINE)
    else:
        return escape(s, PRE_ESCAPED_IN_TERMS)


def escape_quoted_term(s, is_cmdline):
    if is_cmdline:
        return (
            '"'
            + escape(
                s[1 : len(s) - 1], PRE_ESCAPED_IN_QUOTED_TERMS | PRE_ESCAPED_IN_CMDLINE
            )
            + '"'
        )
    else:
        return '"' + escape(s[1 : len(s) - 1], PRE_ESCAPED_IN_QUOTED_TERMS) + '"'


def uppercase_to(s, _):
    return s.replace(" to ", " TO ", 1)


def escape(s, pre_escaped):
    sb = ""
    i = 0
    while i < len(s):
        c = s[i]
        # // These characters are part of the query syntax and must be escaped
        if c == "\\" and i + 1 < len(s) and s[i + 1] in pre_escaped:
            sb += c
            sb += s[i + 1]
            i += 2
        else:
            if (
                c == "\\"
                or c == "+"
                or c == "-"
                or c == "!"
                or c == ":"
                or c == "^"
                or c == "["
                or c == "]"
                or c == "{"
                or c == "}"
                or c == "~"
                or c == "|"
                or c == "&"
                or c == "/"
                or c == " "
            ):
                sb += "\\"
            sb += c
            i += 1
    return sb


def convert_query(query):
    matches = {}
    counter = 1
    query, counter = extract_matches(
        query, QUOTED_PATTERN, matches, counter, escape_quoted_term, True
    )
    query, counter = extract_matches(
        query, RANGE_PATTERN, matches, counter, uppercase_to, False
    )
    query, counter = extract_matches(
        query, TERM_PATTERN, matches, counter, escape_term, True
    )

    # Replace all lowercase or, and, to with upper case
    query = query.replace("or", "OR")
    query = query.replace("and", "AND")

    for key, value in matches.items():
        query = query.replace(key, value, 1)
    return query


if __name__ == "__main__":
    queries = [
        "(ipport:[10 to 20] and ipaddr:1.2.3.4) or"
        ' ((filemod:c:\\program\\ files\\\\ mike.exe)) cmdline:"This is: \\"silly\\""',
        'cmdline:"hi" cmdline:"hello" ##2##',
        'cmdline:"hi\\\\there" cmdline:"hello" cmdline:hi\\\\there filemod:hi\\\\there filemod:"hi\\\\there"',
        "cmdline:hi cmdline:hello ipport:[0 to 10]",
        'os_type:osx or (os_type:MAC or (os_type:"osx") or os_type:windows) or os_type:"MAC" or os_type:MA*',
        'block_status:"ProcessTerminated" or block_status:ProcessTerminated or block_status:"bl ah"',
        '(cmdline:"/c echo" AND cmdline:". pipe\\")',
        "modload:inetpub\\SolarWinds\\bin\\*.dll AND -digsig_result_modload:signed",
        '(parent_name:taskeng.exe AND cmdline:"//E:JScript" or cmdline:"//E:JavaScript" or cmdline:"/E:JavaScript" or cmdline:"//E:vbscript" or cmdline:"/E:JScript" or cmdline:"/E:vbscript" or cmdline:"-E:JScript" or cmdline:"-e:vbscript" or cmdline:"-E:JavaScript"  -cmdline:logmein)',
    ]

    for q in queries:
        print(f"ORIG: {q}")
        print(f"CONVERTED: {convert_query(q)}")
