import logging
from feed_manager.engine.watchlist_engine import WatchlistEngine
from feed_manager.utils.tasks.background_task import BackgroundTask
from pscr_common.utils import get_utcnow_with_tzinfo
from sqlalchemy.exc import IntegrityError

from feed_manager.utils.db import is_unique_violation_exception

_logger = logging.getLogger(__name__)

# blacklist_report = Report(id="L0aeJy37RpOoQHHdvpbcg",
#                           timestamp=datetime_from_sec_timestamp(1557946545),
#                           link=None,
#                           title='Hash Banning',
#                           description='Hash Banning',
#                           severity=10,
#                           iocs_v2=[IOC_V2(id="hashban",
#                                           match_type="equality",
#                                           field="sensor_action_reason",
#                                           values=["HASHBAN"])])

shared_reports = []

# blacklist_watchlist = WatchlistV2(id="naFB2YkvTh1vWhNZGDK8Q",
#                                   name="Hash Banning",
#                                   description="Hash Banning",
#                                   tags_enabled=False,
#                                   alerts_enabled=True,
#                                   report_ids=[blacklist_report.id])

shared_watchlists = []


class SharedWatchlistTask(BackgroundTask):
    def __init__(self, watchlist_engine: WatchlistEngine):
        super(SharedWatchlistTask, self).__init__(
            "shared_watchlist", loop_sec=60, sleep_first=False, execute_once=True
        )
        self.watchlist_engine = watchlist_engine

    async def execute(self, app):
        cur_watchlists = {
            w.id: w for w in await self.watchlist_engine.get_watchlists("shared")
        }
        new_watchlists = {w.id: w for w in shared_watchlists}
        reports = {r.id: r for r in shared_reports}

        # Delete any old watchlists and their associated reports
        for deleted_watchlist_id in set(cur_watchlists.keys()) - set(
            new_watchlists.keys()
        ):
            watchlist = cur_watchlists[deleted_watchlist_id]
            for report_id in watchlist.report_ids:
                await self._delete_report(report_id)
            if await self.watchlist_engine.remove_watchlist(
                "shared", deleted_watchlist_id
            ):
                _logger.info("Deleted shared watchlist: %s", deleted_watchlist_id)

        # Create new watchlists
        for new_watchlist_id in set(new_watchlists.keys()) - set(cur_watchlists.keys()):
            new_watchlist = new_watchlists[new_watchlist_id]
            for report_id in new_watchlist.report_ids:
                await self._add_report(reports[report_id])
            try:
                watchlist = await self.watchlist_engine.add_watchlist(
                    "shared", new_watchlist, get_utcnow_with_tzinfo()
                )
                _logger.info("Added shared watchlist: %s", watchlist)
            except ValueError:
                # This is a race condition when first creating the watchlist in ephemeral environments
                #  when multiple instances start at the same time.
                pass

        # Update existing watchlists
        for watchlist_id in set(new_watchlists.keys()) & set(cur_watchlists.keys()):
            cur_watchlist = cur_watchlists[watchlist_id]
            new_watchlist = new_watchlists[watchlist_id]

            for deleted_report_id in set(cur_watchlist.report_ids) - set(
                new_watchlist.report_ids
            ):
                await self._delete_report(deleted_report_id)
            for report in [
                reports[report_id] for report_id in new_watchlist.report_ids
            ]:
                if report.id in cur_watchlist.report_ids:  # Is this an existing report
                    try:
                        await self.watchlist_engine.update_report("shared", report)
                        _logger.info("Updated shared report: %s", report)
                    except ValueError:
                        # This will happen if the report is already updated in the db
                        pass
                else:
                    await self._add_report(report)

    async def _add_report(self, report):
        try:
            report = await self.watchlist_engine.add_report(
                "shared", report, access="public"
            )
            _logger.info("Added shared report: %s", report)
        except IntegrityError as e:
            if not is_unique_violation_exception(e):
                raise e
            # This is a race condition when first creating the report in ephemeral environments
            #  when multiple feed manager instances start at the same time.
            pass

    async def _delete_report(self, report_id: str):
        if await self.watchlist_engine.delete_report("shared", report_id):
            _logger.info("Deleted shared watchlist report: %s", report_id)
