import logging
import rapidjson

from aiohttp import web
from aiohttp.web_exceptions import HTTPNotFound, HTTPBadRequest

from feed_manager.api.models import Report
from feed_manager.api.models.watchlist_models import Watchlist, ReportSeverity
from feed_manager.engine.report_engine import ReportEngine
from feed_manager.engine.watchlist_engine import WatchlistEngine
from feed_manager.web.route_helpers import make_response_json, get_request_org
from feed_manager.web.auth import Resource
from pscr_common.utils import get_utcnow_with_tzinfo
from pscr_common.web.auth import AuthRouteTableDef, Action
from pscr_common.web.auditlog import call_audit_log

_logger = logging.getLogger(__name__)


def build_route_table(engine: WatchlistEngine) -> AuthRouteTableDef:
    routes = AuthRouteTableDef()

    def _get_required_param(params, key):
        value = params.get(key)
        if not value:
            raise HTTPBadRequest(reason=f"Must provide '{key}'")
        return value

    @routes.get("/healthcheck")
    async def healthcheck(request: web.Request):
        """
        ---
        summary: Healthcheck
        description: Successful response indicates service reachability.
        produces:
        - application/octet-stream
        responses:
            204:
                description: service is available
        """
        request.do_not_log = True

        return make_response_json()

    @routes.post("/v1/watchlist", Resource.watchlists, [Action.create])
    async def create_watchlist(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Create new watchlist
        description: Create a new report or classifier watchlist.  Unique watchlist ID will be generated by the service.  Request must specify `report` or `classifier` but not both.
        consumes:
        - application/json
        produces:
        - application/json
        parameters:
          - in: body
            name: body
            description: Watchlist payload
            required: true
            schema:
              $ref: '#/definitions/Watchlist'
        responses:
            200:
                description: Watchlist created.
                schema:
                  $ref: '#/definitions/Watchlist'
            400:
                description: invalid watchlist request.
        """
        now = get_utcnow_with_tzinfo()
        params = await request.json(loads=rapidjson.loads)
        if "classifier" in params:
            params["classifier"]["watchlist_id"] = WatchlistEngine.gen_watchlist_id()
        elif "report" in params:
            params["report"]["watchlist_id"] = WatchlistEngine.gen_watchlist_id()

        watchlist_request = Watchlist.from_dict(params)

        watchlist = await engine.add_watchlist(
            get_request_org(request), watchlist_request.to_v2(), now
        )
        await call_audit_log(
            request,
            description=f"Created new watchlist, '{watchlist.name}' (ID: {watchlist.id}) (v1)",
        )
        return make_response_json(
            WatchlistEngine.remove_empty_alert_classification_field(watchlist.to_v1())
        )

    @routes.get("/v1/watchlist", Resource.watchlists, [Action.read], allow_head=False)
    async def get_watchlists(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Get all watchlists
        description: Retrieve all watchlists owned by the caller.
        produces:
        - application/json
        parameters: []
        responses:
            200:
                description: Array of watchlists
                schema:
                  properties:
                    results:
                      type: array
                      items:
                        $ref: '#/definitions/Watchlist'
        """
        org_id = get_request_org(request)

        watchlists = await engine.get_watchlists(org_id=org_id)

        return make_response_json(
            {
                "results": [
                    WatchlistEngine.remove_empty_alert_classification_field(
                        watchlist.to_v1()
                    )
                    for watchlist in watchlists
                ]
            }
        )

    @routes.get(
        "/v1/watchlist/{watchlist_id}",
        Resource.watchlists,
        [Action.read],
        allow_head=False,
    )
    async def get_watchlist(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Get watchlist
        description: Retrieve watchlist with `watchlist_id`
        produces:
        - application/json
        parameters:
          - in: path
            name: watchlist_id
            description: ID for watchlist
            required: true
            type: string
        responses:
            200:
                description: Return watchlist
                schema:
                  $ref: '#/definitions/Watchlist'
            404:
                description: Unknown watchlist.
        """
        watchlist_id = request.match_info["watchlist_id"]

        watchlist = (
            await engine.get_watchlists(
                org_id=get_request_org(request), watchlist_id=watchlist_id
            )
        )[0]

        return make_response_json(
            WatchlistEngine.remove_empty_alert_classification_field(watchlist.to_v1())
        )

    @routes.put("/v1/watchlist/{watchlist_id}", Resource.watchlists, [Action.update])
    async def update_watchlist(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Update watchlist
        description: Update watchlist with `watchlist_id`.  This will update the tags and alert status as well as any reports or classifiers attached to the watchlist.  If a field is missing or null (ie tags_enabled) that field will not be updated.  Cannot update report watchlist with empty `report_ids` list.
        produces:
        - application/json
        parameters:
          - in: path
            name: watchlist_id
            description: ID for watchlist
            required: true
            type: string
          - in: body
            name: body
            description: Watchlist payload
            required: true
            schema:
              $ref: '#/definitions/Watchlist'
        responses:
            200:
                description: Return watchlist
                schema:
                  $ref: '#/definitions/Watchlist'
            400:
                description: Unknown watchlist or malformed request.
        """
        now = get_utcnow_with_tzinfo()
        params = await request.json(loads=rapidjson.loads)
        watchlist_request = Watchlist.from_dict(params)

        watchlist = await engine.update_watchlist(
            get_request_org(request), watchlist_request.to_v2(), now
        )

        await call_audit_log(
            request,
            description=f"Updated watchlist, '{watchlist.name}' (ID: {watchlist.id}) (v1)",
        )
        return make_response_json(
            WatchlistEngine.remove_empty_alert_classification_field(watchlist.to_v1())
        )

    @routes.delete("/v1/watchlist/{watchlist_id}", Resource.watchlists, [Action.delete])
    async def remove_watchlist(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Delete watchlist
        description: Remove watchlist with `watchlist_id`.  Existing hits for this watchlist will remain in the system.
        produces:
        - application/octet-stream
        parameters:
          - in: path
            name: watchlist_id
            description: ID for watchlist
            required: true
            type: string
        responses:
            204:
                description: Watchlist deleted
            400:
                description: Unknown watchlist.
        """
        watchlist_id = request.match_info["watchlist_id"]
        org_key = get_request_org(request)

        watchlist_name = await engine.get_watchlist_name(org_key, watchlist_id)

        if not await engine.remove_watchlist(org_key, watchlist_id):
            raise HTTPNotFound(reason=f"Unknown watchlist: {watchlist_id}")

        await call_audit_log(
            request,
            description=f"Deleted watchlist, '{watchlist_name}' (ID: {watchlist_id}) (v1)",
        )
        return make_response_json()

    @routes.get(
        "/v1/watchlist/{watchlist_id}/alert",
        Resource.watchlists,
        [Action.read],
        allow_head=False,
    )
    async def get_watchlist_alert_status(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Get watchlist alert status
        description: Retrieve alert status for watchlist with `watchlist_id`.
        produces:
        - application/json
        parameters:
          - in: path
            name: watchlist_id
            description: ID for watchlist
            required: true
            type: string
        responses:
            200:
                description: Returns alert status
                schema:
                  type: object
                  properties:
                    alert:
                      type: boolean
        """
        watchlist_id = request.match_info["watchlist_id"]
        alert = await engine.get_alert_status(get_request_org(request), watchlist_id)

        return make_response_json({"alert": alert})

    @routes.put(
        "/v1/watchlist/{watchlist_id}/alert", Resource.watchlists, [Action.update]
    )
    async def set_watchlist_alert(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Enable watchlist alerts
        description: Turn on alerts for watchlist with `watchlist_id`.  This is not retroactive for existing watchlist hits.  Future hits will trigger alerts.
        produces:
        - application/json
        parameters:
          - in: path
            name: watchlist_id
            description: ID for watchlist
            required: true
            type: string
        responses:
            200:
                description: Returns alert status
                schema:
                  type: object
                  required:
                  - alert
                  properties:
                    alert:
                      type: boolean
            400:
                description: Unknown watchlist
        """
        watchlist_id = request.match_info["watchlist_id"]
        org_key = get_request_org(request)

        if not await engine.set_alert_status(org_key, watchlist_id):
            raise HTTPNotFound(reason=f"Unknown watchlist: {watchlist_id}")

        watchlist_name = await engine.get_watchlist_name(org_key, watchlist_id)

        await call_audit_log(
            request,
            description=f"Enabled alerting for watchlist, '{watchlist_name}' (ID: {watchlist_id}) (v1)",
        )
        return make_response_json({"alert": True})

    @routes.delete(
        "/v1/watchlist/{watchlist_id}/alert", Resource.watchlists, [Action.delete]
    )
    async def remove_watchlist_alert_status(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Disable watchlist alerts
        description: Turn off alerts for watchlist with `watchlist_id`.  This is not retroactive for existing watchlist alerts.  Future hits will not trigger alerts.
        produces:
        - application/octet-stream
        parameters:
          - in: path
            name: watchlist_id
            description: ID for watchlist
            required: true
            type: string
        responses:
            204:
                description: Returns alert status
            400:
                description: Unknown watchlist
        """
        watchlist_id = request.match_info["watchlist_id"]
        org_key = get_request_org(request)

        if not await engine.remove_alert_status(org_key, watchlist_id):
            raise HTTPNotFound(reason=f"Unknown watchlist: {watchlist_id}")

        watchlist_name = await engine.get_watchlist_name(org_key, watchlist_id)

        await call_audit_log(
            request,
            description=f"Disabled alerting for watchlist, '{watchlist_name}' (ID: {watchlist_id}) (v1)",
        )
        return make_response_json()

    @routes.get(
        "/v1/watchlist/{watchlist_id}/tag",
        Resource.watchlists,
        [Action.read],
        allow_head=False,
    )
    async def get_watchlist_tag_status(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Get watchlist tag status
        description: Retrieve tag status for watchlist with `watchlist_id`.
        produces:
        - application/json
        parameters:
          - in: path
            name: watchlist_id
            description: ID for watchlist
            required: true
            type: string
        responses:
            200:
                description: Returns tag status
                schema:
                  type: object
                  properties:
                    tag:
                      type: boolean
            400:
                description: Unknown watchlist
        """
        watchlist_id = request.match_info["watchlist_id"]
        alert = await engine.get_tag_status(get_request_org(request), watchlist_id)

        return make_response_json({"tag": alert})

    @routes.put(
        "/v1/watchlist/{watchlist_id}/tag", Resource.watchlists, [Action.update]
    )
    async def set_watchlist_tag(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Enable watchlist tags
        description: Turn on tagging for watchlist with `watchlist_id`.  This is not retroactive for existing watchlist matches.  Future matches will trigger event tagging.
        produces:
        - application/json
        parameters:
          - in: path
            name: watchlist_id
            description: ID for watchlist
            required: true
            type: string
        responses:
            200:
                description: Returns tag status
                schema:
                  type: object
                  properties:
                    tag:
                      type: boolean
            400:
                description: Unknown watchlist
        """
        watchlist_id = request.match_info["watchlist_id"]
        org_key = get_request_org(request)

        if not await engine.set_tag_status(org_key, watchlist_id):
            raise HTTPNotFound(reason=f"Unknown watchlist: {watchlist_id}")

        watchlist_name = await engine.get_watchlist_name(org_key, watchlist_id)

        await call_audit_log(
            request,
            description=f"Enabled tagging for watchlist, '{watchlist_name}' (ID: {watchlist_id}) (v1)",
        )
        return make_response_json({"tag": True})

    @routes.delete(
        "/v1/watchlist/{watchlist_id}/tag", Resource.watchlists, [Action.delete]
    )
    async def remove_watchlist_tag_status(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Disable watchlist tags
        description: Turn off tagging for watchlist with `watchlist_id`.  This is not retroactive for existing watchlist tags.  Future matches will not trigger event tagging.
        produces:
        - application/octet-stream
        parameters:
          - in: path
            name: watchlist_id
            description: ID for watchlist
            required: true
            type: string
        responses:
            204:
                description: Tagging stopped
            400:
                description: Unknown watchlist
        """
        watchlist_id = request.match_info["watchlist_id"]
        org_key = get_request_org(request)

        if not await engine.remove_tag_status(org_key, watchlist_id):
            raise HTTPNotFound(reason=f"Unknown watchlist: {watchlist_id}")

        watchlist_name = await engine.get_watchlist_name(org_key, watchlist_id)

        await call_audit_log(
            request,
            description=f"Disabled tagging for watchlist, '{watchlist_name}' (ID: {watchlist_id}) (v1)",
        )
        return make_response_json()

    @routes.get(
        "/v1/report/{report_id}/ignore",
        Resource.watchlists,
        [Action.read],
        allow_head=False,
    )
    async def get_report_ignore(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Get report ignore status
        description: Get current ignore status for report with `report_id`.
        produces:
        - application/json
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
        responses:
            200:
                description: Returns ignore status
                schema:
                  type: object
                  properties:
                    ignored:
                      type: boolean
        """
        report_id = request.match_info["report_id"]
        ignored = await engine.get_report_ignore(get_request_org(request), report_id)

        return make_response_json({"ignored": ignored})

    @routes.get(
        "/v1/report/{report_id}/ioc/{ioc_id}/ignore",
        Resource.watchlists,
        [Action.read],
        allow_head=False,
    )
    async def get_report_ioc_ignore(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Get IOC ignore status
        description: Get current ignore status for IOC `ioc_id` in report `report_id`
        produces:
        - application/json
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
          - in: path
            name: ioc_id
            description: ID for ioc
            required: true
            type: string
        responses:
            200:
                description: Returns ignore status
                schema:
                  type: object
                  properties:
                    ignored:
                      type: boolean
        """
        report_id = request.match_info["report_id"]
        ioc_id = request.match_info["ioc_id"]

        ignored = await engine.get_report_ioc_ignore(
            get_request_org(request), report_id, ioc_id
        )

        return make_response_json({"ignored": ignored})

    @routes.put("/v1/report/{report_id}/ignore", Resource.watchlists, [Action.update])
    async def add_report_ignore(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Ignore report
        description: Report with `report_id` and all contained IOCs will not match future events for any watchlist.
        produces:
        - application/json
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
        responses:
            200:
                description: Returns ignore status
                schema:
                  type: object
                  properties:
                    ignored:
                      type: boolean
        """
        report_id = request.match_info["report_id"]
        await engine.add_ignore_report(get_request_org(request), report_id)
        report_title = await engine.get_report_title(report_id)

        await call_audit_log(
            request,
            description=f"Ignored report, '{report_title}' (ID: {report_id}) (v1)",
        )
        return make_response_json({"ignored": True})

    @routes.put(
        "/v1/report/{report_id}/ioc/{ioc_id}/ignore",
        Resource.watchlists,
        [Action.update],
    )
    async def add_report_ioc_ignore(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Ignore IOC
        description: IOC `ioc_id` for report `report_id` will not match future events for any watchlist.
        produces:
        - application/json
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
          - in: path
            name: ioc_id
            description: ID for ioc
            required: true
            type: string
        responses:
            200:
                description: Returns ignore status
                schema:
                  type: object
                  properties:
                    ignored:
                      type: boolean
        """
        report_id = request.match_info["report_id"]
        ioc_id = request.match_info["ioc_id"]

        await engine.add_ignore_report_ioc(get_request_org(request), report_id, ioc_id)
        report_title = await engine.get_report_title(report_id)

        await call_audit_log(
            request,
            description=f"Ignored IOC, (ID: {ioc_id}) from report, '{report_title}' (ID: {report_id}) (v1)",
        )
        return make_response_json({"ignored": True})

    @routes.delete(
        "/v1/report/{report_id}/ignore", Resource.watchlists, [Action.delete]
    )
    async def remove_report_ignore(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Re-activate report
        description: Report with `report_id` and all contained IOCs will match future events for all watchlists.  This is not retroactive for events that occured while the report was ignored.
        produces:
        - application/octet-stream
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
        responses:
            204:
                description: Report is active
            400:
                description: Unknown report
        """
        report_id = request.match_info["report_id"]

        report_title = await engine.get_report_title(report_id)

        await engine.remove_ignore_report(get_request_org(request), report_id)

        await call_audit_log(
            request,
            description=f"Enabled report, '{report_title}' (ID: {report_id}) (v1)",
        )
        return make_response_json()

    @routes.delete(
        "/v1/report/{report_id}/ioc/{ioc_id}/ignore",
        Resource.watchlists,
        [Action.delete],
    )
    async def remove_report_ioc_ignore(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Re-activate IOC
        description: IOC `ioc_id` for report `report_id` and will match future events for all watchlists.  This is not retroactive for events that occured while the IOC was ignored.
        produces:
        - application/octet-stream
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
          - in: path
            name: ioc_id
            description: ID for ioc
            required: true
            type: string
        responses:
            204:
                description: Ignore removed - IOC is active
            400:
                description: Unknown report/ioc
        """
        report_id = request.match_info["report_id"]
        ioc_id = request.match_info["ioc_id"]

        await engine.remove_ignore_report_ioc(
            get_request_org(request), report_id, ioc_id
        )

        report_title = await engine.get_report_title(report_id)

        await call_audit_log(
            request,
            description=f"Enabled IOC, (ID: {ioc_id}) from report, '{report_title}' (ID: {report_id}) (v1)",
        )
        return make_response_json()

    @routes.get("/v1/severity", Resource.watchlists, [Action.read], allow_head=False)
    async def get_report_severity_overrides(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Get custom report severities
        description: Return all custom report severities.  Custom report severities effect all watchlists.
        produces:
        - application/json
        parameters: []
        responses:
            200:
                description: Returns list of report severities
                schema:
                  properties:
                    results:
                      type: array
                      items:
                        $ref: '#/definitions/ReportSeverity'
        """
        return make_response_json(
            {
                "results": await engine.get_report_severity_overrides(
                    get_request_org(request)
                )
            }
        )

    @routes.get(
        "/v1/severity/report/{report_id}",
        Resource.watchlists,
        [Action.read],
        allow_head=False,
    )
    async def get_report_severity_override(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Get custom severity for report
        description: Return custom severity for `report_id`.  This will return 404 error if custom severity doesn't exist.
        produces:
        - application/json
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
        responses:
            200:
                description: Returns severity.  (null if not set)
                schema:
                  $ref: '#/definitions/ReportSeverity'
            404:
                description: No override for report
        """
        report_id = request.match_info["report_id"]
        report_severity = await engine.get_report_severity_override(
            get_request_org(request), report_id
        )
        if not report_severity:
            raise HTTPNotFound(reason=f"No override for report: {report_id}")

        return make_response_json(report_severity)

    @routes.put("/v1/severity/report/{report_id}", Resource.watchlists, [Action.update])
    async def set_report_severity_override(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Set custom report severity
        description: Adjust the severity of report with `report_id`.  This will effect all watchlists.
        consumes:
        - application/json
        produces:
        - application/json
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
          - in: body
            name: body
            description: report severity
            required: true
            schema:
              properties:
                severity:
                  type: integer
        responses:
            200:
                description: Returns severity.
                schema:
                  $ref: '#/definitions/ReportSeverity'
        """
        report_id = request.match_info["report_id"]

        params = await request.json(loads=rapidjson.loads)
        severity = int(_get_required_param(params, "severity"))

        await engine.add_report_severity_override(
            get_request_org(request), report_id, severity
        )

        report_title = await engine.get_report_title(report_id)

        await call_audit_log(
            request,
            description=f"Set severity to {severity} for report, '{report_title}' (ID: {report_id}) (v1)",
        )
        return make_response_json(ReportSeverity(report_id, severity))

    @routes.delete(
        "/v1/severity/report/{report_id}", Resource.watchlists, [Action.delete]
    )
    async def remove_report_severity_override(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Remove custom report severity
        description: Remove custom severity for report with `report_id`
        produces:
        - application/octet-stream
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
        responses:
            204:
                description: Severity override removed
        """
        report_id = request.match_info["report_id"]

        await engine.remove_report_severity_override(
            get_request_org(request), report_id
        )

        report_title = await engine.get_report_title(report_id)

        await call_audit_log(
            request,
            description=f"Removed severity from report, '{report_title}' (ID: {report_id}) (v1)",
        )
        return make_response_json()

    @routes.post("/v1/report", Resource.watchlists, [Action.create])
    async def create_report(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Create new report
        description: Add a new watchlist report.  This service will generate a unique report id.  This report will be private to the caller.  IOCs will be converted to IOC_V2.
        consumes:
        - application/json
        produces:
        - application/json
        parameters:
          - in: body
            name: body
            description: Report payload
            required: true
            schema:
              $ref: '#/definitions/Report'
        responses:
            200:
                description: Report created
                schema:
                  $ref: '#/definitions/Report'
            400:
                description: invalid report request.
        """
        report_id = ReportEngine.gen_report_id()

        params = await request.json(loads=rapidjson.loads)

        org_id = get_request_org(request)

        params["id"] = report_id

        report = await engine.add_report(org_id, Report.from_dict(params))

        await call_audit_log(
            request, description=f"Created report {report.title}({report_id}) (v1)"
        )
        return make_response_json(report)

    @routes.put("/v1/report/{report_id}", Resource.watchlists, [Action.update])
    async def update_report(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Update a report.
        description: Update report with `report_id`.  This will replace all fields in the report.  Any fields not provided in the request will be remove from the report.  All IOCs will be converted to IOC_V2.  The report must be owned by the caller.
        consumes:
        - application/json
        produces:
        - application/json
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
          - in: body
            name: body
            description: Report payload
            required: true
            schema:
              $ref: '#/definitions/Report'
        responses:
            200:
                description: Report updated
                schema:
                  $ref: '#/definitions/Report'
            400:
                description: invalid report request.
            404:
                description: report id not found
        """
        report_id = request.match_info["report_id"]

        params = await request.json(loads=rapidjson.loads)

        org_id = get_request_org(request)

        params["id"] = report_id

        report = await engine.update_report(org_id, Report.from_dict(params))

        await call_audit_log(
            request,
            description=f"Updated report, '{report.title}' (ID: {report_id}) (v1)",
        )
        return make_response_json(report)

    @routes.get(
        "/v1/report/{report_id}", Resource.watchlists, [Action.read], allow_head=False
    )
    async def get_report(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Get report
        description: Retrieve report with `report_id`.  The report must be owned by the caller.
        produces:
        - application/json
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
        responses:
            200:
                description: Report
                schema:
                  $ref: '#/definitions/Report'
            404:
                description: report id not found
        """
        report_id = request.match_info["report_id"]

        org_id = get_request_org(request)

        report = await engine.get_report(org_id, report_id)

        return make_response_json(report)

    @routes.delete("/v1/report/{report_id}", Resource.watchlists, [Action.delete])
    async def remove_report(request: web.Request):
        """
        ---
        tags:
        - v1
        summary: Remove report
        description: Remove report with `report_id`.  The report must be owned by the caller.
        produces:
        - application/json
        parameters:
          - in: path
            name: report_id
            description: ID for report
            required: true
            type: string
        responses:
            204:
                description: Report deleted
            404:
                description: report id not found
        """
        report_id = request.match_info["report_id"]

        org_id = get_request_org(request)

        report_title = await engine.get_report_title(report_id)

        if not await engine.delete_report(org_id, report_id):
            raise HTTPNotFound(reason=f"Unknown report: {report_id}")

        await call_audit_log(
            request,
            description=f"Deleted report, '{report_title}' (ID: {report_id}) (v1)",
        )
        return make_response_json()

    return routes
